@@ -15,7 +15,6 @@ from string import *
 import subprocess
 import math
 import copy
-import threading
 
 from Tkinter import *
 import tkMessageBox
@@ -25,7 +24,7 @@ import threading
 import time
 
 from xasyVersion import xasyVersion
-from xasyCodeEditor import *
+import xasyCodeEditor
 from xasy2asy import *
 import xasyFile
 import xasyOptions
@@ -58,13 +57,9 @@ class xasyMainWin:
     self.createWidgets()
     self.resetGUI()
     if sys.platform[:3] == "win":
-      site="http://effbot.org/downloads/PIL-1.1.6.win32-py2.5.exe"
-      if sys.version_info[0] != 2 or sys.version_info[1] != 5:
-        tkMessageBox.showerror("Failed Dependencies","Xasy requires Python 2.5 or 2.5.1. Please install http://www.python.org/ftp/python/2.5.1/python-2.5.1.msi")
-        self.parent.destroy()
-        sys.exit(1)
+      site="http://effbot.org/downloads/PIL-1.1.7.win32-py2.6.exe"
     else:
-      site="http://effbot.org/downloads/Imaging-1.1.6.tar.gz after applying the patch http://asymptote.svn.sourceforge.net/viewvc/asymptote/trunk/asymptote/patches/TkAlpha-Imaging-1.1.6.patch" 
+      site="http://effbot.org/downloads/Imaging-1.1.7.tar.gz" 
     if not PILAvailable:
       tkMessageBox.showerror("Failed Dependencies","An error occurred loading the required PIL library. Please install "+site)
       self.parent.destroy()
@@ -149,6 +144,7 @@ class xasyMainWin:
     self.exportMenu.add_command(label="PDF...",command=self.exportPDF,underline=0)
     self.exportMenu.add_command(label="GIF...",command=self.exportGIF,underline=0)
     self.exportMenu.add_command(label="PNG...",command=self.exportPNG,underline=1)
+    self.exportMenu.add_command(label="SVG...",command=self.exportSVG,underline=0)
     self.fileMenu.add_cascade(label="Export",menu=self.exportMenu,underline=1)
     self.fileMenu.add_separator()
 
@@ -725,6 +721,9 @@ class xasyMainWin:
   def exportPNG(self):
     self.exportFile(self.filename,"png")
 
+  def exportSVG(self):
+    self.exportFile(self.filename,"svg")
+
   def exportFile(self,inFile, outFormat):
     if(not self.testOrAcquireLock()):
       return
@@ -748,9 +747,8 @@ class xasyMainWin:
       return
     fullname = os.path.abspath(outfilename)
     outName = os.path.basename(outfilename)
-    dirname = os.path.dirname(fullname)
-    command = xasyOptions.options['asyPath']+" -f %s -o %s %s"%(outFormat,fullname,inFile)
-    saver = subprocess.Popen(split(command),stdin=PIPE,stdout=PIPE,stderr=PIPE)
+    command=[xasyOptions.options['asyPath'],"-f"+outFormat,"-o"+fullname,inFile]
+    saver = subprocess.Popen(command,stdin=PIPE,stdout=PIPE,stderr=PIPE)
     saver.wait()
     if saver.returncode != 0:
       tkMessageBox.showerror("Export Error","Export Error:\n"+saver.stdout.read()+saver.stderr.read())
@@ -838,21 +836,25 @@ class xasyMainWin:
   def toolTextCmd(self):
     self.updateSelectedButton(self.toolTextButton)
   def toolAsyCmd(self):
-    self.updateSelectedButton(self.toolSelectButton)
     # ignore the command if we are too busy to process it
     if not self.testOrAcquireLock():
       return
     self.updateSelectedButton(self.toolSelectButton)
     self.clearSelection()
     self.clearHighlight()
-    self.addItemToFile(xasyScript(self.mainCanvas))
     self.unbindGlobalEvents()
-    text = xasyCodeEditor(self.parent,"// enter your code here").getText()
+    try:
+      self.getNewText("// enter your code here")
+    except Exception, e:
+      tkMessageBox.showerror('xasy Error',e.message)
+    else:
+      self.addItemToFile(xasyScript(self.mainCanvas))
+      text = self.newText
+      self.undoRedoStack.add(addScriptAction(self,self.fileItems[-1]))
+      self.fileItems[-1].setScript(text)
+      self.fileItems[-1].drawOnCanvas(self.mainCanvas,self.magnification)
+      self.bindItemEvents(self.fileItems[-1])
     self.bindGlobalEvents()
-    self.undoRedoStack.add(addScriptAction(self,self.fileItems[-1]))
-    self.fileItems[-1].setScript(text)
-    self.fileItems[-1].drawOnCanvas(self.mainCanvas,self.magnification)
-    self.bindItemEvents(self.fileItems[-1])
     self.releaseLock()
   def toolRaiseCmd(self):
     if(not self.testOrAcquireLock()):
@@ -1148,21 +1150,42 @@ class xasyMainWin:
     self.mainCanvas.delete(ID)
     self.populatePropertyList()
 
+  def scriptEditThread(self,oldText):
+    try:
+      self.newText = xasyCodeEditor.getText(oldText)
+    except:
+      self.newText = -1
+
+  def getNewText(self,oldText):
+    editThread = threading.Thread(target=self.scriptEditThread,args=(oldText,))
+    editThread.start()
+    while editThread.isAlive():
+      time.sleep(0.05)
+      self.parent.update()
+    editThread.join()
+    if type(self.newText)==type(-1):
+      self.newText = ''
+      raise Exception('Error launching external editor. Please check xasy options.')
+
   def itemEdit(self,item):
     # are we too busy?
     if not self.testOrAcquireLock():
       return
     self.updateSelectedButton(self.toolSelectButton)
     if isinstance(item,xasyScript):
-      oldText = item.script
       self.unbindGlobalEvents()
-      newText = xasyCodeEditor(self.parent,item.script).getText()
+      oldText = item.script
+      try:
+        self.getNewText(oldText)
+      except Exception,e:
+        tkMessageBox.showerror('xasy Error',e.message)
+      else:
+        if self.newText != oldText:
+          self.undoRedoStack.add(editScriptAction(self,item,self.newText,oldText))
+          item.setScript(self.newText)
+          item.drawOnCanvas(self.mainCanvas,self.magnification)
+          self.bindItemEvents(item)
       self.bindGlobalEvents()
-      if newText != oldText:
-        self.undoRedoStack.add(editScriptAction(self,item,newText,oldText))
-        item.setScript(newText)
-        item.drawOnCanvas(self.mainCanvas,self.magnification)
-        self.bindItemEvents(item)
     elif isinstance(item,xasyText):
       theText = tkSimpleDialog.askstring(title="Xasy - Text",prompt="Enter text to display:",initialvalue=item.label.text,parent=self.parent)
       if theText != None and theText != "":
@@ -1614,7 +1637,6 @@ class xasyMainWin:
 
   def resetOptions(self):
     xasyOptions.setDefaults()
-    xasyOptions.save()
     self.applyOptions()
 
   def applyPenWidth(self):
