@@ -7,8 +7,6 @@
 #ifndef ARRAYOP_H
 #define ARRAYOP_H
 
-#include <typeinfo>
-
 #include "util.h"
 #include "stack.h"
 #include "array.h"
@@ -63,28 +61,6 @@ void arrayArrayOp(vm::stack *s)
   s->push(c);
 }
 
-template<class T>
-inline void arrayNegate(vm::stack *s)
-{
-  array *a=pop<array*>(s);
-  size_t size=checkArray(a);
-  array *c=new array(size);
-  for(size_t i=0; i < size; i++)
-    (*c)[i]=-read<T>(a,i);
-  s->push(c);
-}
-
-template<>
-inline void arrayNegate<Int>(vm::stack *s)
-{
-  array *a=pop<array*>(s);
-  size_t size=checkArray(a);
-  array *c=new array(size);
-  for(size_t i=0; i < size; i++)
-    (*c)[i]=Negate(read<Int>(a,i),i);
-  s->push(c);
-}
-
 template<class T>
 void sumArray(vm::stack *s)
 {
@@ -161,6 +137,61 @@ void binopArray3(vm::stack *s)
   s->push(m);
 }
 
+template<class T, class U, template <class S> class op>
+void array2Op(vm::stack *s)
+{
+  U b=pop<U>(s);
+  array *a=pop<array*>(s);
+  size_t size=checkArray(a);
+  array *c=new array(size);
+  for(size_t i=0; i < size; ++i) {
+    array *ai=read<array*>(a,i);
+    size_t aisize=checkArray(ai);
+    array *ci=new array(aisize);
+    (*c)[i]=ci;
+    for(size_t j=0; j < aisize; j++)
+      (*ci)[j]=op<T>()(read<T>(ai,j),b,0);
+  }
+  s->push(c);
+}
+
+template<class T, class U, template <class S> class op>
+void opArray2(vm::stack *s)
+{
+  array *a=pop<array*>(s);
+  T b=pop<T>(s);
+  size_t size=checkArray(a);
+  array *c=new array(size);
+  for(size_t i=0; i < size; ++i) {
+    array *ai=read<array*>(a,i);
+    size_t aisize=checkArray(ai);
+    array *ci=new array(aisize);
+    (*c)[i]=ci;
+    for(size_t j=0; j < aisize; j++)
+      (*ci)[j]=op<U>()(read<U>(ai,j),b,0);
+  }
+  s->push(c);
+}
+
+template<class T, template <class S> class op>
+void array2Array2Op(vm::stack *s)
+{
+  array *b=pop<array*>(s);
+  array *a=pop<array*>(s);
+  size_t size=checkArrays(a,b);
+  array *c=new array(size);
+  for(size_t i=0; i < size; ++i) {
+    array *ai=read<array*>(a,i);
+    array *bi=read<array*>(b,i);
+    size_t aisize=checkArrays(ai,bi);
+    array *ci=new array(aisize);
+    (*c)[i]=ci;
+    for(size_t j=0; j < aisize; j++)
+      (*ci)[j]=op<T>()(read<T>(ai,j),read<T>(bi,j),0);
+  }
+  s->push(c);
+}
+
 template<class T>
 bool Array2Equals(vm::stack *s)
 {
@@ -195,6 +226,24 @@ void array2NotEquals(vm::stack *s)
   s->push(!Array2Equals<T>(s));
 }
 
+template<class T>
+void diagonal(vm::stack *s)
+{
+  array *a=pop<array*>(s);
+  size_t n=checkArray(a);
+  array *c=new array(n);
+  for(size_t i=0; i < n; ++i) {
+    array *ci=new array(n);
+    (*c)[i]=ci;
+    for(size_t j=0; j < i; ++j)
+      (*ci)[j]=T();
+    (*ci)[i]=read<T>(a,i);
+    for(size_t j=i+1; j < n; ++j)
+      (*ci)[j]=T();
+  }
+  s->push(c);
+}
+
 template<class T>
 struct compare {
   bool operator() (const vm::item& a, const vm::item& b)
@@ -238,26 +287,24 @@ void sortArray2(vm::stack *s)
   s->push(c);
 }
 
-// Search a sorted ordered array of n elements to find an interval containing
-// a given key. Returns n-1 if the key is greater than or equal to the last
-// element, -1 if the key is less than the first element, and otherwise the
-// index corresponding to the left-hand endpoint of the matching interval. 
+// Search a sorted ordered array a of n elements for key, returning the index i
+// if a[i] <= key < a[i+1], -1 if key is less than all elements of a, or
+// n-1 if key is greater than or equal to the last element of a.
 template<class T>
 void searchArray(vm::stack *s)
 {
   T key=pop<T>(s);
   array *a=pop<array*>(s);
-  Int size=(Int) a->size();
-  if(size == 0) {s->push(0); return;}
-  if(key < read<T>(a,0)) {s->push(-1); return;}
-  Int u=size-1;
-  if(key >= read<T>(a,u)) {s->push(u); return;}
-  Int l=0;
+  size_t size= a->size();
+  if(size == 0 || key < read<T>(a,0)) {s->push(-1); return;}
+  size_t u=size-1;
+  if(key >= read<T>(a,u)) {s->push((Int) u); return;}
+  size_t l=0;
         
   while (l < u) {
-    Int i=(l+u)/2;
-    if(read<T>(a,i) <= key && key < read<T>(a,i+1)) {s->push(i); return;}
+    size_t i=(l+u)/2;
     if(key < read<T>(a,i)) u=i;
+    else if(key < read<T>(a,i+1)) {s->push((Int) i); return;}
     else l=i+1;
   }
   s->push(0);
@@ -435,9 +482,93 @@ void arrayFunc(vm::stack *s)
   s->push(c);
 }
 
+template <class T, class S, T (*func)(S)>
+void arrayFunc2(vm::stack *s) 
+{
+  array *a=pop<array*>(s);
+  size_t size=checkArray(a);
+  array *c=new array(size);
+  for(size_t i=0; i < size; ++i) {
+    array *ai=read<array*>(a,i);
+    size_t aisize=checkArray(ai);
+    array *ci=new array(aisize);
+    (*c)[i]=ci;
+    for(size_t j=0; j < aisize; j++)
+    (*ci)[j]=func(read<S>(ai,j));
+  }
+  s->push(c);
+}
+
 vm::array *Identity(Int n);
 camp::triple operator *(const vm::array& a, const camp::triple& v);
 camp::triple multshiftless(const vm::array& t, const camp::triple& v);
+double norm(double *a, size_t n);
+double norm(camp::triple *a, size_t n);
+
+inline size_t checkdimension(const vm::array *a, size_t dim)
+{
+  size_t size=checkArray(a);
+  if(dim && size != dim) {
+    ostringstream buf;
+    buf << "array of length " << dim << " expected";
+    vm::error(buf);
+  }
+  return size;
+}
+
+template<class T>
+inline void copyArrayC(T* &dest, const vm::array *a, size_t dim=0,
+                       GCPlacement placement=NoGC)
+{
+  size_t size=checkdimension(a,dim);
+  dest=(placement == NoGC) ? new T[size] : new(placement) T[size];
+  for(size_t i=0; i < size; i++) 
+    dest[i]=vm::read<T>(a,i);
+}
+
+template<class T, class A>
+inline void copyArrayC(T* &dest, const vm::array *a, T (*cast)(A),
+                       size_t dim=0, GCPlacement placement=NoGC)
+{
+  size_t size=checkdimension(a,dim);
+  dest=(placement == NoGC) ? new T[size] : new(placement) T[size];
+  for(size_t i=0; i < size; i++) 
+    dest[i]=cast(vm::read<A>(a,i));
+}
+
+template<typename T>
+inline vm::array* copyCArray(const size_t n, const T* p,
+                             GCPlacement placement=NoGC)
+{
+  vm::array* a = new vm::array(n);
+  for(size_t i=0; i < n; ++i) (*a)[i] = p[i];
+  return a;
+}
+
+template<class T>
+inline void copyArray2C(T* &dest, const vm::array *a, bool square=true,
+                        size_t dim2=0, GCPlacement placement=NoGC)
+{
+  size_t n=checkArray(a);
+  size_t m=(square || n == 0) ? n : checkArray(vm::read<vm::array*>(a,0));
+  if(n > 0 && dim2 && m != dim2) {
+    ostringstream buf;
+    buf << "second matrix dimension must be " << dim2;
+    vm::error(buf);
+  }
+  
+  dest=(placement == NoGC) ? new T[n*m] : new(placement) T[n*m];
+  for(size_t i=0; i < n; i++) {
+    vm::array *ai=vm::read<vm::array*>(a,i);
+    size_t aisize=checkArray(ai);
+    if(aisize == m) {
+      T *desti=dest+i*m;
+      for(size_t j=0; j < m; j++) 
+        desti[j]=vm::read<T>(ai,j);
+    } else
+      vm::error(square ? "matrix must be square" : "matrix must be rectangular");
+  }
+}
 
 } // namespace run
 
