@@ -34,6 +34,18 @@ void reportWarning(const string& desc)
   em.sync();
 }
 
+void reportFatal(const string& desc)
+{
+  em.fatal(vm::getPos());
+  em << desc;
+  em.sync();
+  em.statusError();
+  try {
+    throw quit(); 
+  } catch(handled_error) {
+  }
+}
+
 void reportError(const ostringstream& desc)
 {
   reportError(desc.str());
@@ -44,4 +56,9 @@ void reportWarning(const ostringstream& desc)
   reportWarning(desc.str());
 }
   
+void reportFatal(const ostringstream& desc)
+{
+  reportFatal(desc.str());
+}
+  
 } // namespace camp
