@@ -34,7 +34,7 @@ using namespace types;
 // Defined in dec.cc.
 varEntry *makeVarEntry(position pos, coenv &e, record *r, types::ty *t);
 
-bool definesImplicitConstructor(coenv &e, record *r, varEntry *v, symbol *id)
+bool definesImplicitConstructor(coenv &e, record *r, varEntry *v, symbol id)
 {
   if (id == symbol::initsym &&
       r != 0 &&
@@ -63,7 +63,7 @@ void transConstructorBody(position pos, coenv &e, record *r, varEntry *init)
   // Initialize the object.  a=new Foo;
   newRecordExp::transFromTyEntry(pos, e, new tyEntry(r, 0, 0, position()));
   v->encode(WRITE, pos, e.c);
-  e.c.encode(inst::pop);
+  e.c.encodePop();
 
   // Push the args onto the stack.
   size_t numArgs=init->getSignature()->getNumFormals();
@@ -92,8 +92,11 @@ varEntry *constructorFromInitializer(position pos, coenv &e, record *r,
 
   types::function *ft=new types::function(r, init->getSignature());
  
+  ostringstream out;
+  ft->printVar(out, symbol::trans("<constructor>"));
+
   // Create a new function environment.
-  coder fc = e.c.newFunction(ft);
+  coder fc = e.c.newFunction(pos, out.str(), ft);
   coenv fe(fc,e.e);
 
   // Translate the function.
@@ -111,7 +114,7 @@ varEntry *constructorFromInitializer(position pos, coenv &e, record *r,
   // Save it into a varEntry.
   varEntry *v=makeVarEntry(pos, e, r, ft);
   v->encode(WRITE, pos, e.c);
-  e.c.encode(inst::pop);
+  e.c.encodePop();
 
   return v;
 }
