@@ -13,74 +13,64 @@
 #include "psfile.h"
 #include "util.h"
 
+using vm::array;
+using vm::read;
+
 namespace camp {
 
-double PatternLength(double arclength, const std::vector<double>& pat,
-		     const path& p, double penwidth)
+double PatternLength(double arclength, const array& pat,
+                     bool cyclic, double penwidth)
 {
   double sum=0.0;
       
   size_t n=pat.size();
   for(unsigned i=0; i < n; i ++)
-    sum += pat[i]*penwidth;
+    sum += read<double>(pat,i)*penwidth;
   
   if(sum == 0.0) return 0.0;
   
   if(n % 2 == 1) sum *= 2.0; // On/off pattern repeats after 2 cycles.
       
+  double pat0=read<double>(pat,0);
   // Fix bounding box resolution problem. Example:
   // asy -f pdf testlinetype; gv -scale -2 testlinetype.pdf
-  if(!p.cyclic() && pat[0] == 0) sum += 1.0e-3*penwidth;
+  if(!cyclic && pat0 == 0) sum += 1.0e-3*penwidth;
       
-  double terminator=((p.cyclic() && arclength >= 0.5*sum) ? 0.0 : 
-		     pat[0]*penwidth);
+  double terminator=(cyclic && arclength >= 0.5*sum) ? 0.0 : pat0*penwidth;
   int ncycle=(int)((arclength-terminator)/sum+0.5);
 
   return (ncycle >= 1 || terminator >= 0.75*arclength) ? 
     ncycle*sum+terminator : 0.0;
 }
 
-void drawPath::adjustdash(pen& pen0)
+bool isdashed(pen& p) {
+  const LineType *linetype=p.linetype();
+  size_t n=linetype->pattern.size();
+  return n > 0;
+}
+
+pen adjustdash(pen& p, double arclength, bool cyclic)
 {
+  pen q=p;
   // Adjust dash sizes to fit arclength; also compensate for linewidth.
-  string stroke=pen0.stroke();
-  
-  if(!stroke.empty()) {
-    double penwidth=pen0.linetype().scale ? pen0.width() : 1.0;
+  const LineType *linetype=q.linetype();
+  size_t n=linetype->pattern.size();
+    
+  if(n > 0) {
+    double penwidth=linetype->scale ? q.width() : 1.0;
     double factor=penwidth;
     
-    std::vector<double> pat;
-    
-    istringstream ibuf(stroke);
-    double l;
-    while(ibuf >> l) {
-      if(l < 0) l=0;
-      pat.push_back(l);
-    }
-      
-    size_t n=pat.size();
-    
-    if(pen0.linetype().adjust) {
-      double arclength=p.arclength();
-      if(arclength) {
-	if(n == 0) return;
-      
-	double denom=PatternLength(arclength,pat,p,penwidth);
-	if(denom != 0.0) factor *= arclength/denom;
-      }
+    if(linetype->adjust && arclength) {
+      double denom=PatternLength(arclength,linetype->pattern,cyclic,penwidth);
+      if(denom != 0.0) factor *= arclength/denom;
     }
     
-    factor=max(factor,0.1);
-    
-    ostringstream buf;
-    buf.setf(std::ios::fixed);
-    for(unsigned i=0; i < n; i++)
-      buf << pat[i]*factor << " ";
-    pen0.setstroke(buf.str());
-    pen0.setoffset(pen0.linetype().offset*factor);
+    if(factor != 1.0)
+      q.adjust(max(factor,0.1));
   }
+  return q;
 }
-  
+ 
 // Account for square or extended pen cap contributions to bounding box.
 void cap(bbox& b, double t, path p, pen pentype) {
   transform T=pentype.getTransform();  
@@ -97,14 +87,14 @@ void cap(bbox& b, double t, path p, pen pentype) {
   pair z=shift(T)*p.point(t);
   
   switch(pentype.cap()) {
-  case 0:
+    case 0:
     {
       pair d=rotate(v)*pair(x,y)*h;
       b += z+d;
       b += z-d;
       break;
     }
-  case 2:
+    case 2:
     {
       transform R=rotate(v);
       double w=(xx*yy-xy*yx)/y;
@@ -114,12 +104,12 @@ void cap(bbox& b, double t, path p, pen pentype) {
       b += z+dm;
       b += z-dp;
       b += z-dm;
-    break;
+      break;
     }
   }
 }
 
-void drawPath::bounds(bbox& b, iopipestream&, boxvector&, bboxlist&)
+void drawPathPenBase::strokebounds(bbox& b, const path& p)
 {
   Int l=p.length();
   if(l < 0) return;
@@ -140,23 +130,26 @@ void drawPath::bounds(bbox& b, iopipestream&, boxvector&, bboxlist&)
 bool drawPath::draw(psfile *out)
 {
   Int n = p.size();
-  if (n == 0 || pentype.invisible() || pentype.width() == 0.0)
+  if (n == 0 || pentype.invisible())
     return true;
 
-  pen pen0=pentype;
-  adjustdash(pen0);
+  pen q = isdashed(pentype) ? adjustdash(pentype,p.arclength(),p.cyclic()) :
+                              pentype;
 
   penSave(out);
   penTranslate(out);
 
-  out->write(p);
+  if(n > 1)
+    out->write(p);
+  else
+    out->dot(p,q);
 
   penConcat(out);
-
-  out->setpen(pen0);
   
-  out->stroke();
-
+  out->setpen(q);
+  
+  out->stroke(q,n == 1);
+  
   penRestore(out);
 
   return true;
