@@ -19,15 +19,18 @@ public:
   
   virtual ~drawPath() {}
 
-  void bounds(bbox& b, iopipestream&, boxvector&, bboxlist&);
+  void bounds(bbox& b, iopipestream&, boxvector&, bboxlist&) {
+    strokebounds(b,p);
+  }
 
+  bool svg() {return true;}
+  
   bool draw(psfile *out);
 
-  virtual void adjustdash(pen &);
-
   drawElement *transformed(const transform& t);
 };
 
+pen adjustdash(pen& p, double arclength, bool cyclic);
 }
 
 #endif
