@@ -5,12 +5,17 @@
 #ifdef HAVE_LIBFFTW3
 #include "fftw++.h"
 
-ifstream fftw::ifWisdom;
-ofstream fftw::ofWisdom;
+namespace fftwpp {
+
+std::ifstream fftw::ifWisdom;
+std::ofstream fftw::ofWisdom;
 bool fftw::Wise=false;
+const double fftw::twopi=2.0*acos(-1.0);
 
 // User settings:
-unsigned int fftw::effort=FFTW_PATIENT;
+unsigned int fftw::effort=FFTW_MEASURE;
 const char *fftw::WisdomName=".wisdom";
 
+}
+
 #endif
