@@ -1,9 +1,9 @@
 /******
-  * fileio.h
-  * Tom Prince and John Bowman 2004/05/10
-  *
-  * Handle input/output
-  ******/
+ * fileio.h
+ * Tom Prince and John Bowman 2004/05/10
+ *
+ * Handle input/output
+ ******/
 
 #ifndef FILEIO_H
 #define FILEIO_H
@@ -30,7 +30,7 @@
 #include "process.h"
 
 namespace vm {
-  extern bool indebugger;  
+extern bool indebugger;  
 }
 
 namespace camp {
@@ -38,23 +38,36 @@ namespace camp {
 extern string tab;
 extern string newline;
   
+enum Mode {NOMODE,INPUT,OUTPUT,UPDATE,BINPUT,BOUTPUT,BUPDATE,XINPUT,XOUTPUT,
+           XUPDATE};
+
+static const string FileModes[]={"none",
+                                 "input","output","ouptut(update=false)",
+                                 "binput","boutput","boutput(update=false)",
+                                 "xinput","xoutput","xoutput(update=false)"};
+
 class file : public gc {
 protected:  
   string name;
+  bool check;      // Check whether input file exists.
+  Mode type;
+  
   Int nx,ny,nz;    // Array dimensions
   bool linemode;   // Array reads will stop at eol instead of eof.
   bool csvmode;    // Read comma-separated values.
   bool wordmode;   // Delimit strings by white space instead of eol.
   bool singlereal; // Read/write single-precision XDR/binary reals.
   bool singleint;  // Read/write single-precision XDR/binary ints.
+  bool signedint;  // Read/write signed XDR/binary ints.
+  
   bool closed;     // File has been closed.
-  bool check;      // Check whether input file exists.
   bool standard;   // Standard input/output
   bool binary;     // Read in binary mode.
   
   bool nullfield;  // Used to detect a final null field in csv+line mode.
   string whitespace;
-  size_t index;	   // Terminator index.
+  size_t index;    // Terminator index.
+
 public: 
 
   bool Standard() {return standard;}
@@ -62,7 +75,7 @@ public:
   void standardEOF() {
 #if defined(HAVE_LIBREADLINE) && defined(HAVE_LIBCURSES)
     cout << endl;
-#endif	
+#endif  
   }
   
   template<class T>
@@ -70,13 +83,13 @@ public:
     if(standard) {
       int c;
       if(cin.eof())
-	standardEOF();
+        standardEOF();
       else {
-	cin.clear();
-	while((c=cin.peek()) != EOF) {
-	  cin.ignore();
-	  if(c == '\n') break;
-	}
+        cin.clear();
+        while((c=cin.peek()) != EOF) {
+          cin.ignore();
+          if(c == '\n') break;
+        }
       }
     }
   }
@@ -88,11 +101,11 @@ public:
   
   void dimension(Int Nx=-1, Int Ny=-1, Int Nz=-1) {nx=Nx; ny=Ny; nz=Nz;}
   
-  file(const string& name, bool check=true, bool binary=false,
+  file(const string& name, bool check=true, Mode type=NOMODE, bool binary=false,
        bool closed=false) : 
-    name(name), linemode(false), csvmode(false),
-    singlereal(false), singleint(true),
-    closed(closed), check(check), standard(name.empty()),
+    name(name), check(check), type(type), linemode(false), csvmode(false),
+    singlereal(false), singleint(true), signedint(true),
+    closed(closed), standard(name.empty()),
     binary(binary), nullfield(false), whitespace("") {dimension();}
   
   virtual void open() {}
@@ -107,8 +120,6 @@ public:
   
   virtual ~file() {}
 
-  virtual const char* Mode() {return "";}
-
   bool isOpen() {
     if(closed) {
       ostringstream buf;
@@ -119,7 +130,7 @@ public:
     }
     return true;
   }
-		
+                
   string filename() {return name;}
   virtual bool eol() {return false;}
   virtual bool nexteol() {return false;}
@@ -133,10 +144,12 @@ public:
   virtual size_t tell() {return 0;}
   virtual void seek(Int, bool=true) {}
   
+  string FileMode() {return FileModes[type];}
+  
   void unsupported(const char *rw, const char *type) {
     ostringstream buf;
-    buf << rw << " of type " << type << " not supported in " << Mode()
-	<< " mode.";
+    buf << rw << " of type " << type << " not supported in " << FileMode()
+        << " mode.";
     reportError(buf);
   }
   
@@ -144,7 +157,7 @@ public:
   void nowrite(const char *type) {unsupported("Write",type);}
   
   virtual void Read(bool&) {noread("bool");}
-  virtual void Read(Int&) {noread("Int");}
+  virtual void Read(Int&) {noread("int");}
   virtual void Read(double&) {noread("real");}
   virtual void Read(float&) {noread("real");}
   virtual void Read(pair&) {noread("pair");}
@@ -154,7 +167,7 @@ public:
   virtual void readwhite(string&) {noread("string");}
   
   virtual void write(bool) {nowrite("bool");}
-  virtual void write(Int) {nowrite("Int");}
+  virtual void write(Int) {nowrite("int");}
   virtual void write(double) {nowrite("real");}
   virtual void write(const pair&) {nowrite("pair");}
   virtual void write(const triple&) {nowrite("triple");}
@@ -182,12 +195,12 @@ public:
       if(standard) clear();
       if(errorstream::interrupt) throw interrupted();
       else {
-	ignoreComment(val);
-	val=T();
-	if(!nullfield)
-	  Read(val);
-	csv();
-	whitespace="";
+        ignoreComment(val);
+        val=T();
+        if(!nullfield)
+          Read(val);
+        csv();
+        whitespace="";
       }
     }
   }
@@ -214,6 +227,9 @@ public:
   
   void SingleInt(bool b) {singleint=b;}
   bool SingleInt() {return singleint;}
+  
+  void SignedInt(bool b) {signedint=b;}
+  bool SignedInt() {return signedint;}
 };
 
 class ifile : public file {
@@ -225,43 +241,40 @@ protected:
   bool comma;
   
 public:
-  ifile(const string& name, char comment, bool check=true,
-	std::ios::openmode mode=std::ios::in) :
-    file(name,check), stream(NULL), fstream(NULL), comment(comment), mode(mode),
-    comma(false) {
-    stream=&cin;
-  }
+  ifile(const string& name, char comment, bool check=true, Mode type=INPUT, 
+        std::ios::openmode mode=std::ios::in) :
+    file(name,check,type), stream(&cin), fstream(NULL), comment(comment),
+    mode(mode), comma(false) {}
   
   // Binary file
-  ifile(const string& name, bool check=true,
-	std::ios::openmode mode=std::ios::in) :
-    file(name,check,true), mode(mode) {}
+  ifile(const string& name, bool check=true, Mode type=BINPUT,
+        std::ios::openmode mode=std::ios::in) :
+    file(name,check,type,true), mode(mode) {}
   
   ~ifile() {close();}
   
   void open() {
     if(standard) {
       if(mode & std::ios::binary) 
-	reportError("Cannot open standard input in binary mode");
+        reportError("Cannot open standard input in binary mode");
       stream=&cin;
     } else {
+      if(mode & std::ios::out)
+        name=outpath(name);
       stream=fstream=new std::fstream(name.c_str(),mode);
       if(mode & std::ios::out) {
-	checkLocal(name);
-	if(error()) {
-	  delete fstream;
-	  std::ofstream f(name.c_str());
-	  f.close();
-	}
-	stream=fstream=new std::fstream(name.c_str(),mode);
+        if(error()) {
+          delete fstream;
+          std::ofstream f(name.c_str());
+          f.close();
+          stream=fstream=new std::fstream(name.c_str(),mode);
+        }
       }
       index=processData().ifile.add(fstream);
       if(check) Check();
     }
   }
   
-  const char* Mode() {return "input";}
-  
   bool eol();
   bool nexteol();
   
@@ -314,7 +327,7 @@ public:
 class iofile : public ifile {
 public:
   iofile(const string& name, char comment=0) : 
-    ifile(name,true,comment,std::ios::in | std::ios::out) {}
+    ifile(name,comment,true,UPDATE,std::ios::in | std::ios::out) {}
 
   Int precision(Int p) {
     return p == 0 ? stream->precision() : stream->precision(p);
@@ -343,28 +356,25 @@ protected:
   std::ofstream *fstream;
   std::ios::openmode mode;
 public:
-  ofile(const string& name, std::ios::openmode mode=std::ios::trunc) :
-    file(name), stream(NULL), fstream(NULL), mode(mode) {
-    stream=&cout;
-  }
+  ofile(const string& name, Mode type=OUTPUT,
+        std::ios::openmode mode=std::ios::trunc) : 
+    file(name,true,type), stream(&cout), fstream(NULL), mode(mode) {}
   
   ~ofile() {close();}
   
   void open() {
     if(standard) {
       if(mode & std::ios::binary) 
-	reportError("Cannot open standard output in binary mode");
+        reportError("Cannot open standard output in binary mode");
       stream=&cout;
     } else {
-      checkLocal(name);
+      name=outpath(name);
       stream=fstream=new std::ofstream(name.c_str(),mode | std::ios::trunc);
       index=processData().ofile.add(fstream);
       Check();
     }
   }
   
-  const char* Mode() {return "output";}
-
   bool text() {return true;}
   bool eof() {return stream->eof();}
   bool error() {return stream->fail();}
@@ -413,22 +423,27 @@ public:
 
 class ibfile : public ifile {
 public:
-  ibfile(const string& name, bool check=true,
-	 std::ios::openmode mode=std::ios::in) : 
-    ifile(name,check,mode | std::ios::binary) {}
+  ibfile(const string& name, bool check=true, Mode type=BINPUT,
+         std::ios::openmode mode=std::ios::in) : 
+    ifile(name,check,type,mode | std::ios::binary) {}
   template<class T>
   void iread(T& val) {
     val=T();
-    fstream->read((char *) &val,sizeof(T));
+    if(fstream) fstream->read((char *) &val,sizeof(T));
   }
   
   void Read(bool& val) {iread(val);}
   void Read(Int& val) {
-    if(singleint) {int ival; iread(ival); val=ival;}
-    else iread(val);
+    if(signedint) {
+      if(singleint) {int ival; iread(ival); val=ival;}
+      else iread(val);
+    } else {
+      if(singleint) {unsigned ival; iread(ival); val=Intcast(ival);}
+      else {unsignedInt ival; iread(ival); val=Intcast(ival);}
+    }
   }
   void Read(char& val) {iread(val);}
-  void Read(string& val) {iread(val);}
+  void Read(string& val) {char c; iread(c); val=c;}
   
   void Read(double& val) {
     if(singlereal) {float fval; iread(fval); val=fval;}
@@ -438,20 +453,25 @@ public:
   
 class iobfile : public ibfile {
 public:
-  iobfile(const string& name) : ibfile(name,true,std::ios::in | std::ios::out)
-  {}
+  iobfile(const string& name) : 
+    ibfile(name,true,BUPDATE,std::ios::in | std::ios::out) {}
 
   void flush() {if(fstream) fstream->flush();}
   
   template<class T>
   void iwrite(T val) {
-    fstream->write((char *) &val,sizeof(T));
+    if(fstream) fstream->write((char *) &val,sizeof(T));
   }
   
   void write(bool val) {iwrite(val);}
   void write(Int val) {
-    if(singleint) iwrite(intcast(val));
-    else iwrite(val);
+    if(signedint) {
+      if(singleint) iwrite(intcast(val));
+      else iwrite(val);
+    } else {
+      if(singleint) iwrite(unsignedcast(val));
+      else iwrite(unsignedIntcast(val));
+    }
   }
   void write(const string& val) {iwrite(val);}
   void write(const pen& val) {iwrite(val);}
@@ -475,17 +495,22 @@ public:
   
 class obfile : public ofile {
 public:
-  obfile(const string& name) : ofile(name,std::ios::binary) {}
+  obfile(const string& name) : ofile(name,BOUTPUT,std::ios::binary) {}
 
   template<class T>
   void iwrite(T val) {
-    fstream->write((char *) &val,sizeof(T));
+    if(fstream) fstream->write((char *) &val,sizeof(T));
   }
   
   void write(bool val) {iwrite(val);}
   void write(Int val) {
-    if(singleint) iwrite(intcast(val));
-    else iwrite(val);
+    if(signedint) {
+      if(singleint) iwrite(intcast(val));
+      else iwrite(val);
+    } else {
+      if(singleint) iwrite(unsignedcast(val));
+      else iwrite(unsignedIntcast(val));
+    }
   }
   void write(const string& val) {iwrite(val);}
   void write(const pen& val) {iwrite(val);}
@@ -515,9 +540,9 @@ protected:
   xdr::ioxstream *fstream;
   xdr::xios::open_mode mode;
 public:
-  ixfile(const string& name, bool check=true,
-	 xdr::xios::open_mode mode=xdr::xios::in) :
-    file(name,check,true), fstream(NULL), mode(mode) {}
+  ixfile(const string& name, bool check=true, Mode type=XINPUT,
+         xdr::xios::open_mode mode=xdr::xios::in) :
+    file(name,check,type,true), fstream(NULL), mode(mode) {}
 
   void open() {
     fstream=new xdr::ioxstream(name.c_str(),mode);
@@ -537,8 +562,6 @@ public:
   
   ~ixfile() {close();}
   
-  const char* Mode() {return "xinput";}
-  
   bool eof() {return fstream ? fstream->eof() : true;}
   bool error() {return fstream ? fstream->fail() : true;}
 
@@ -559,10 +582,12 @@ public:
   }
   
   void Read(Int& val) {
-    if(singleint) {int ival=0; *fstream >> ival; val=ival;}
-    else {
-      val=0;
-      *fstream >> val;
+    if(signedint) {
+      if(singleint) {int ival=0; *fstream >> ival; val=ival;}
+      else {val=0; *fstream >> val;}
+    } else {
+      if(singleint) {unsigned ival=0; *fstream >> ival; val=Intcast(ival);}
+      else {unsignedInt ival=0; *fstream >> ival; val=Intcast(ival);}
     }
   }
   void Read(double& val) {
@@ -589,13 +614,18 @@ public:
 
 class ioxfile : public ixfile {
 public:
-  ioxfile(const string& name) : ixfile(name,true,xdr::xios::out) {}
+  ioxfile(const string& name) : ixfile(name,true,XUPDATE,xdr::xios::out) {}
 
   void flush() {if(fstream) fstream->flush();}
   
   void write(Int val) {
-    if(singleint) *fstream << intcast(val);
-    else *fstream << val;
+    if(signedint) {
+      if(singleint) *fstream << intcast(val);
+      else *fstream << val;
+    } else {
+      if(singleint) *fstream << unsignedcast(val);
+      else *fstream << unsignedIntcast(val);
+    }
   }
   void write(double val) {
     if(singlereal) *fstream << (float) val;
@@ -615,10 +645,10 @@ public:
 class oxfile : public file {
   xdr::oxstream *fstream;
 public:
-  oxfile(const string& name) : file(name), fstream(NULL) {}
+  oxfile(const string& name) : file(name,true,XOUTPUT), fstream(NULL) {}
 
   void open() {
-    fstream=new xdr::oxstream((checkLocal(name),name.c_str()),xdr::xios::trunc);
+    fstream=new xdr::oxstream(outpath(name).c_str(),xdr::xios::trunc);
     index=processData().oxfile.add(fstream);
     Check();
   }
@@ -635,8 +665,6 @@ public:
   
   ~oxfile() {close();}
   
-  const char* Mode() {return "xoutput";}
-  
   bool eof() {return fstream ? fstream->eof() : true;}
   bool error() {return fstream ? fstream->fail() : true;}
   void clear() {if(fstream) fstream->clear();}
@@ -657,8 +685,13 @@ public:
   }
   
   void write(Int val) {
-    if(singleint) *fstream << intcast(val);
-    else *fstream << val;
+    if(signedint) {
+      if(singleint) *fstream << intcast(val);
+      else *fstream << val;
+    } else {
+      if(singleint) *fstream << unsignedcast(val);
+      else *fstream << unsignedIntcast(val);
+    }
   }
   void write(double val) {
     if(singlereal) *fstream << (float) val;
