@@ -46,13 +46,18 @@
 
 using namespace settings;
 
-errorstream em;
 using interact::interactive;
 
 namespace run {
 void purge();
 }
   
+#ifdef PROFILE
+namespace vm {
+extern void dumpProfile();
+};
+#endif
+
 #ifdef HAVE_LIBSIGSEGV
 void stackoverflow_handler (int, stackoverflow_context_t)
 {
@@ -102,20 +107,6 @@ void interruptHandler(int)
   em.Interrupt(true);
 }
 
-// Run the config file.
-void doConfig(string file) 
-{
-  bool autoplain=getSetting<bool>("autoplain");
-  bool listvariables=getSetting<bool>("listvariables");
-  if(autoplain) Setting("autoplain")=false; // Turn off for speed.
-  if(listvariables) Setting("listvariables")=false;
-
-  runFile(file);
-
-  if(autoplain) Setting("autoplain")=true;
-  if(listvariables) Setting("listvariables")=true;
-}
-
 struct Args 
 {
   int argc;
@@ -155,6 +146,10 @@ void *asymain(void *A)
       }
   }
 
+#ifdef PROFILE
+  vm::dumpProfile();
+#endif
+
   if(getSetting<bool>("wait")) {
     int status;
     while(wait(&status) > 0);
@@ -179,7 +174,7 @@ int main(int argc, char *argv[])
   Args args(argc,argv);
 #ifdef HAVE_GL
   gl::glthread=getSetting<bool>("threads");
-#ifdef HAVE_LIBPTHREAD
+#if HAVE_LIBPTHREAD
   
   if(gl::glthread) {
     pthread_t thread;
