@@ -13,6 +13,7 @@
 
 #include "stack.h"
 #include "mod.h"
+#include "triple.h"
 
 namespace run {
 
@@ -76,6 +77,16 @@ struct times {
   T operator() (T x, T y, size_t=0) {return x*y;}
 };
 
+template <>
+struct times<camp::triple> {
+  camp::triple operator() (double x, camp::triple y, size_t=0) {return x*y;}
+};
+
+template <typename T>
+struct timesR {
+  T operator () (T y, double x, size_t=0) {return x*y;}
+};
+
 extern void dividebyzero(size_t i=0);  
 extern void integeroverflow(size_t i=0);  
   
@@ -87,6 +98,11 @@ struct divide {
   }
 };
 
+template <>
+struct divide<camp::triple> {
+  camp::triple operator() (camp::triple x, double y, size_t=0) {return x/y;}
+};
+
 inline bool validInt(double x) {
   return x > Int_MIN-0.5 && x < Int_MAX+0.5;
 }
@@ -127,8 +143,9 @@ struct times<Int> {
   Int operator() (Int x, Int y, size_t i=0) {
     if(y == 0) return 0;
     if(y < 0) {y=-y; x=-x;}
-    if(x > Int_MAX/y || x < Int_MIN/y)
-       integeroverflow(i);
+    if((y > int_MAX || x > int_MAX/(int) y || x < int_MIN/(int) y) && 
+       (x > Int_MAX/y || x < Int_MIN/y))
+      integeroverflow(i);
     return x*y;
   }
 };
@@ -174,11 +191,11 @@ T pow(T x, Int y)
   if(y == 0) return 1.0;
   if(x == 0.0 && y > 0) return 0.0;
   if(y < 0) {y=-y; x=1/x;}
-	
+        
   T r=1.0;
   for(;;) {
     if(y & 1) r *= x;
-    if((y >>= 1) == 0)	return r;
+    if((y >>= 1) == 0)  return r;
     x *= x;
   }
 }
@@ -201,14 +218,14 @@ struct power<Int> {
       if(x == 0) return 0;
       Int r = 1;
       for(;;) {
-	if(p & 1) {
-	  if(r > Int_MAX/x) integeroverflow(i);
-	  r *= x;
-	}
-	if((p >>= 1) == 0)
-	  return sign*r;
-	if(x > Int_MAX/x) integeroverflow(i);
-	x *= x;
+        if(p & 1) {
+          if(r > Int_MAX/x) integeroverflow(i);
+          r *= x;
+        }
+        if((p >>= 1) == 0)
+          return sign*r;
+        if(x > Int_MAX/x) integeroverflow(i);
+        x *= x;
       }
     } else {
       if(x == 1) return sign;
@@ -239,6 +256,40 @@ struct max {
   T operator() (T x, T y, size_t=0) {return x > y ? x : y;}
 };
 
+template<class T>
+inline T Min(T a, T b)
+{
+  return (a < b) ? a : b;
+}
+
+template<class T>
+inline T Max(T a, T b)
+{
+  return (a > b) ? a : b;
+}
+
+template <typename T>
+struct minbound {
+  camp::pair operator() (camp::pair z, camp::pair w) {
+    return camp::pair(Min(z.getx(),w.getx()),Min(z.gety(),w.gety()));
+  }
+  camp::triple operator() (camp::triple u, camp::triple v) {
+    return camp::triple(Min(u.getx(),v.getx()),Min(u.gety(),v.gety()),
+                        Min(u.getz(),v.getz()));
+  }
+};
+
+template <typename T>
+struct maxbound {
+  camp::pair operator() (camp::pair z, camp::pair w) {
+    return camp::pair(Max(z.getx(),w.getx()),Max(z.gety(),w.gety()));
+  }
+  camp::triple operator() (camp::triple u, camp::triple v) {
+    return camp::triple(Max(u.getx(),v.getx()),Max(u.gety(),v.gety()),
+                        Max(u.getz(),v.getz()));
+  }
+};
+
 template <double (*func)(double)>
 void realReal(vm::stack *s) 
 {
@@ -260,7 +311,7 @@ void interp(vm::stack *s)
   double t=vm::pop<double>(s);
   T b=vm::pop<T>(s);
   T a=vm::pop<T>(s);
-  s->push(a+t*(b-a));
+  s->push((1-t)*a+t*b);
 }
 
 } // namespace run
