@@ -37,23 +37,23 @@ public:
   
   friend pair operator+ (const pair& z, const pair& w)
   {
-    return pair(z.x + w.x, z.y + w.y);
+    return pair(z.x+w.x,z.y+w.y);
   }
 
   friend pair operator- (const pair& z, const pair& w)
   {
-    return pair(z.x - w.x, z.y - w.y);
+    return pair(z.x-w.x,z.y-w.y);
   }
 
   friend pair operator- (const pair& z)
   {
-    return pair(-z.x, -z.y);
+    return pair(-z.x,-z.y);
   }
 
   // Complex multiplication
   friend pair operator* (const pair& z, const pair& w)
   {
-    return pair(z.x*w.x - z.y*w.y, z.x*w.y + w.x*z.y);
+    return pair(z.x*w.x-z.y*w.y,z.x*w.y+w.x*z.y);
   }
 
   const pair& operator+= (const pair& w)
@@ -104,7 +104,7 @@ public:
 
     double t = 1.0 / (w.x*w.x + w.y*w.y);
     return pair(t*(z.x*w.x + z.y*w.y),
-		t*(-z.x*w.y + w.x*z.y));
+                t*(-z.x*w.y + w.x*z.y));
   }
 
   friend bool operator== (const pair& z, const pair& w)
@@ -154,6 +154,11 @@ public:
     return pair(z.x,-z.y);
   }
   
+  friend double dot(const pair& z, const pair& w)
+  {
+    return z.x*w.x+z.y*w.y;
+  }
+  
 // Return the principal branch of the square root (non-negative real part).
   friend pair Sqrt(const pair& z) {
     double mag=z.length();
