@@ -12,9 +12,12 @@ inline RGBAColour rgba(pen p) {
   return RGBAColour(p.red(),p.green(),p.blue(),p.opacity());
 }
   
+static const double inches=72;
+static const double cm=inches/2.54;
+
 class prcfile : public oPRCFile {
 public:  
-  prcfile(string name) : oPRCFile(name.c_str()) {}
+  prcfile(string name) : oPRCFile(name.c_str(),10.0/cm) {} // Use bp.
 };
 
 } //namespace camp
