@@ -16,6 +16,7 @@
 // contexts.
 
 PRIMITIVE(void,Void,void)
+PRIMITIVE(inferred,Inferred,var)
 /* null is not a primitive type. */
 #ifdef PRIMERROR
 PRIMITIVE(error,Error,<error>)
@@ -30,6 +31,7 @@ PRIMITIVE(triple,Triple,triple)
 PRIMITIVE(transform,Transform,transform)
 PRIMITIVE(guide,Guide,guide)
 PRIMITIVE(path,Path,path)
+PRIMITIVE(path3,Path3,path3)
 PRIMITIVE(cycleToken,CycleToken,cycleToken)
 PRIMITIVE(tensionSpecifier,TensionSpecifier,tensionSpecifier)
 PRIMITIVE(curlSpecifier,CurlSpecifier,curlSpecifier)
