@@ -59,11 +59,61 @@ using absyntax::block;
 
 mem::stack<processDataStruct *> processDataStack;
 
+// Exception-safe way to push and pop the process data.
+class withProcessData {
+  // Do not let this object be dynamically allocated.
+  void *operator new(size_t);
+
+  processDataStruct *pd_ptr;
+public:
+
+  withProcessData(processDataStruct *pd_ptr) : pd_ptr(pd_ptr)
+  {
+    processDataStack.push(pd_ptr);
+  }
+
+  ~withProcessData()
+  {
+    assert(processDataStack.top() == pd_ptr);
+    processDataStack.pop();
+  }
+};
+
 processDataStruct &processData() {
   assert(!processDataStack.empty());
   return *processDataStack.top();
 }
 
+// A process environment.  Basically this just serves as short-hand to start a
+// new global environment (genv) and new process data at the same time.  When
+// it goes out of scope, the process data is popped off the stack.  This also
+// ensures that the data is popped even if an exception is thrown.
+class penv {
+  genv *_ge;
+  processDataStruct _pd;
+
+  // Do not let this object be dynamically allocated.
+  void *operator new(size_t);
+
+public:
+  penv() : _ge(0), _pd() {
+    // Push the processData first, as it needs to be on the stack before genv
+    // is initialized.
+    processDataStack.push(&_pd);
+    _ge = new genv;
+  }
+
+  virtual ~penv() {
+    processDataStack.pop();
+    delete _ge;
+  }
+
+  genv &ge() { return *_ge; }
+
+  processDataStruct *pd() { return &_pd; }
+};
+
+
 void init(bool resetpath=true)
 {
   vm::indebugger=false;
@@ -146,8 +196,8 @@ struct icore {
 public:
 
   // preRun and postRun are the optional activities that take place before and
-  // after running the code specified.  They can be overridden by derived class
-  // that wish different behaviour.
+  // after running the code specified.  They can be overridden by a derived
+  // class that wishes different behaviour.
   virtual void preRun(coenv &e, istack &s) {
     if(getSetting<bool>("autoplain"))
       runAutoplain(e,s);
@@ -167,13 +217,13 @@ public:
     try {
       if(purge) run::purge();
       
-      genv ge;
-      env base_env(ge);
-      coder base_coder;
+      penv pe;
+      env base_env(pe.ge());
+      coder base_coder(nullPos, "icore::doRun");
       coenv e(base_coder,base_env);
 
       vm::interactiveStack s;
-      s.setInitMap(ge.getInitMap());
+      s.setInitMap(pe.ge().getInitMap());
       s.setEnvironment(&e);
 
       preRun(e,s);
@@ -188,6 +238,7 @@ public:
     } catch(std::bad_alloc&) {
       outOfMemory();
     } catch(quit) {
+      // Exception to quit running the current code. Nothing more to do.
     } catch(handled_error) {
       em.statusError();
     }
@@ -200,18 +251,10 @@ public:
   virtual void process(bool purge=false) {
     if (!interactive && getSetting<bool>("parseonly"))
       doParse();
-    else {
-      // This is not done in preRun as it is not an optional step.
-      processDataStruct data;
-      processDataStack.push(&data);
-      
-      if (getSetting<bool>("listvariables"))
-        doList();
-      else
-        doRun(purge);
-
-      processDataStack.pop();
-    }
+    else if (getSetting<bool>("listvariables"))
+      doList();
+    else
+      doRun(purge);
   }
 };
 
@@ -253,8 +296,8 @@ public:
   void doList() {
     block *tree=getTree();
     if (tree) {
-      genv ge;
-      record *r=tree->transAsFile(ge, symbol::trans(getName()));
+      penv pe;
+      record *r=tree->transAsFile(pe.ge(), symbol::trans(getName()));
       r->e.list(r);
     }
   }
@@ -269,17 +312,10 @@ public:
     }
   }
 
-  void doRun(transMode tm=TRANS_NORMAL) {
+  void doExec(transMode tm=TRANS_NORMAL) {
     // Don't prepare an environment to run the code if there isn't any code.
-    if (getTree()) {
-      // This is not done in preRun as it is not an optional step.
-      processDataStruct data;
-      processDataStack.push(&data);
-
+    if (getTree())
       icore::doRun(false,tm);
-
-      processDataStack.pop();
-    }
   }
 };
 
@@ -307,6 +343,15 @@ public:
   }
 };
 
+void printGreeting(bool interactive) {
+  if(!getSetting<bool>("quiet")) {
+    cout << "Welcome to " << PROGRAM << " version " << VERSION << SVN_REVISION;
+    if(interactive)
+      cout << " (to view the manual, type help)";
+    cout << endl;
+  }
+}
+
 class ifile : public itree {
   string filename;
   string outname;
@@ -338,6 +383,7 @@ public:
   }
 
   void process(bool purge=false) {
+    if(verbose > 1) printGreeting(false);
     try {
       init();
     } catch(handled_error) {
@@ -355,13 +401,6 @@ public:
   }
 };
 
-void printGreeting() {
-  if(!getSetting<bool>("quiet"))
-    cout << "Welcome to " << PROGRAM << " version " << VERSION << SVN_REVISION
-         << " (to view the manual, type help)" << endl;
-}
-
-
 // Add a semi-colon terminator, if one is not there.
 string terminateLine(const string line) {
   return (!line.empty() && *(line.rbegin())!=';') ? (line+";") : line;
@@ -613,7 +652,6 @@ class iprompt : public icore {
     if (cl.simple()) {
       // Don't store exit commands in the history file.
       interact::deleteLastLine();
-
       running=false;
       return true;
     }
@@ -630,9 +668,7 @@ class iprompt : public icore {
       running=false;
       restart=true;
       startline="";
-
       run::purge();
-
       return true;
     }
     else return false;
@@ -646,13 +682,19 @@ class iprompt : public icore {
     else return false;
   }
 
-
-  bool input(commandLine cl) {
-    string prefix="include ";
-    string line=prefix+cl.rest;
+  bool erase(commandLine cl) {
     running=false;
     restart=true;
-    startline=line;
+    if (cl.simple())
+      startline="erase()";
+    else startline=cl.line;
+    return true;
+  }
+
+  bool input(commandLine cl) {
+    running=false;
+    restart=true;
+    startline="include "+cl.rest;
     return true;
   }
 
@@ -666,6 +708,7 @@ class iprompt : public icore {
     ADDCOMMAND(q,q);
     ADDCOMMAND(exit,exit);
     ADDCOMMAND(reset,reset);
+    ADDCOMMAND(erase,erase);
     ADDCOMMAND(help,help);
     ADDCOMMAND(input,input);
 
@@ -807,8 +850,8 @@ public:
     }
   }
 
-  void process() {
-    printGreeting();
+  void process(bool purge=false) {
+    printGreeting(true);
     interact::init_interactive();
     try {
       setPath("",true);
@@ -843,13 +886,13 @@ void processPrompt() {
 }
 
 void runCode(absyntax::block *code) {
-  icode(code).doRun();
+  icode(code).doExec();
 }
 void runString(const string& s, bool interactiveWrite) {
-  istring(s).doRun(interactiveWrite ? TRANS_INTERACTIVE : TRANS_NORMAL);
+  istring(s).doExec(interactiveWrite ? TRANS_INTERACTIVE : TRANS_NORMAL);
 }
 void runFile(const string& filename) {
-  ifile(filename).doRun();
+  ifile(filename).doExec();
 }
 void runPrompt() {
   iprompt().doRun();
@@ -866,18 +909,82 @@ void runPromptEmbedded(trans::coenv &e, istack &s) {
 }
 
 void doUnrestrictedList() {
-  processDataStruct data;
-  processDataStack.push(&data);
-  
-  genv ge;
-  env base_env(ge);
-  coder base_coder;
+  penv pe;
+  env base_env(pe.ge());
+  coder base_coder(nullPos, "doUnrestictedList");
   coenv e(base_coder,base_env);
 
   if (getSetting<bool>("autoplain"))
     absyntax::autoplainRunnable()->trans(e);
 
-  processDataStack.pop();
   e.e.list(0);
 }
 
+// Environment class used by external programs linking to the shared library.
+class fullenv : public gc {
+  penv pe;
+  env base_env;
+  coder base_coder;
+  coenv e;
+
+  vm::interactiveStack s;
+
+public:
+  fullenv() :
+    pe(), base_env(pe.ge()), base_coder(nullPos, "fullenv"),
+    e(base_coder, base_env), s()
+  {
+    s.setInitMap(pe.ge().getInitMap());
+    s.setEnvironment(&e);
+
+    // TODO: Add way to not run autoplain.
+    runAutoplain(e, s);
+  }
+
+  coenv &getCoenv()
+  {
+    return e;
+  }
+
+  void runRunnable(runnable *r)
+  {
+    assert(!em.errors()); // TODO: Decide how to handle prior errors.
+
+    try {
+      { withProcessData token(pe.pd()); 
+        ::runRunnable(r, e, s, TRANS_INTERACTIVE);
+      }
+    } catch(std::bad_alloc&) {
+      // TODO: give calling application useful message.
+      cerr << "out of memory" << endl;
+    } catch (quit) {
+      // I'm not sure whether this counts as successfully running the code or
+      // not.
+      cerr << "quit exception" << endl;
+    } catch (handled_error) {
+      // Normally, this is the result of an error that changes the return code
+      // of the free-standing asymptote program.
+      // An error should probably be reported to the application calling the
+      // asymptote library.
+      cerr << "handled error" << endl;
+    }
+
+    em.clear();
+  }
+};
+
+fullenv &getFullEnv()
+{
+  static fullenv fe;
+  return fe;
+}
+
+coenv &coenvInOngoingProcess()
+{
+  return getFullEnv().getCoenv();
+}
+
+void runInOngoingProcess(runnable *r)
+{
+  getFullEnv().runRunnable(r);
+}
